@@ -16,25 +16,18 @@ namespace vcsn
     char fill = os.fill('0');
     switch (c)
       {
-      case -1:   os << "<end-of-file>"; break;
+      case EOF:  os << "<end-of-file>"; break;
       case '\\': os << "\\\\";          break;
       case '\n': os << "\\n";           break;
       default:
-        if (0 <= c && c <= 0177 && std::isprint(c))
+        if (0 <= c && c <= 127 && std::isprint(c))
           {
             if (special && strchr(special, c))
               os << '\\';
             os << uint8_t(c);
           }
         else
-          // Turn into an unsigned, otherwise if c has its highest bit
-          // set, it will be interpreted as a negative char, which
-          // will be mapped to a negative int.  So for instance c =
-          // 255 is mapped to \xFFFFFFF instead of \xFF.
-          //
-          // This happens only when char is "signed char".  Which is
-          // common.
-          os << "\\x" << std::setw(2) << int(uint8_t(c));
+          os << "\\x" << std::setw(2) << c;
         break;
       }
     os.fill(fill);
@@ -42,12 +35,16 @@ namespace vcsn
     return os;
   }
 
-  std::string
-  str_escape(const int c, const char* special)
+  std::ostream&
+  str_escape(std::ostream& os, char c, const char* special)
   {
-    std::ostringstream o;
-    str_escape(o, c, special);
-    return o.str();
+    // Turn into an unsigned, otherwise if c has its highest bit set,
+    // it will be interpreted as a negative char, which will be mapped
+    // to a negative int.  So for instance c = 255 is mapped to
+    // \xFFFFFFF instead of \xFF.
+    //
+    // This happens only when char is "signed char".  Which is common.
+    return str_escape(os, int(uint8_t(c)), special);
   }
 
   std::ostream&
@@ -57,12 +54,4 @@ namespace vcsn
       str_escape(o, c, special);
     return o;
   }
-
-  std::string
-  str_escape(const std::string& s, const char* special)
-  {
-    std::ostringstream o;
-    str_escape(o, s, special);
-    return o.str();
-  }
 }
