@@ -1,12 +1,8 @@
 #include <cctype>
-#include <map>
 
 #include <vcsn/core/rat/identities.hh>
 #include <vcsn/misc/builtins.hh>
 #include <vcsn/misc/getargs.hh>
-#include <vcsn/misc/initializer_list.hh>
-#include <vcsn/misc/map.hh>
-#include <vcsn/misc/stream.hh>
 
 namespace vcsn
 {
@@ -14,12 +10,39 @@ namespace vcsn
   namespace rat
   {
 
+    identities::identities(const std::string& s)
+    {
+      static const auto map = getarg<identities::ids_t>
+        {
+          "identities",
+          {
+            {"agressive",    identities::agressive},
+            {"associative",  identities::associative},
+            {"auto",         identities::deflt},
+            {"binary",       "trivial"},
+            {"default",      identities::deflt},
+            {"distributive", identities::distributive},
+            {"linear",       identities::linear},
+            {"none",         identities::none},
+            {"series",       "distributive"},
+            {"trivial",      identities::trivial},
+          }
+        };
+      ids_ = map[s];
+    }
+
+    identities::identities(const char* cp)
+      : identities(std::string{cp})
+    {}
+
     std::string to_string(identities i)
     {
       switch (i.ids())
         {
         case identities::associative:
           return "associative";
+        case identities::agressive:
+          return "agressive";
         case identities::linear:
           return "linear";
         case identities::distributive:
@@ -37,35 +60,12 @@ namespace vcsn
       return os << to_string(i);
     }
 
-    static std::istream& operator>>(std::istream& is, identities::ids_t& ids)
+    std::istream& operator>>(std::istream& is, identities& ids)
     {
-      static const auto map = getarg<identities::ids_t>
-        {
-          "identities",
-          {
-            {"associative",  identities::associative},
-            {"auto",         identities::deflt},
-            {"binary",       "trivial"},
-            {"default",      identities::deflt},
-            {"distributive", identities::distributive},
-            {"linear",       identities::linear},
-            {"none",         identities::none},
-            {"series",       "distributive"},
-            {"trivial",      identities::trivial},
-          }
-        };
       std::string buf;
       while (is && isalnum(is.peek()))
         buf += is.get();
-      ids = map[buf];
-      return is;
-    }
-
-    std::istream& operator>>(std::istream& is, identities& ids)
-    {
-      identities::ids_t id;
-      is >> id;
-      ids = identities{id};
+      ids = identities{buf};
       return is;
     }
 
@@ -73,7 +73,5 @@ namespace vcsn
     {
       return std::max(i1, i2);
     }
-
   } // namespace rat
-
 } // namespace vcsn
