@@ -12,8 +12,6 @@
 # undef label
 #endif
 
-#if defined __clang__
-# if 3 <= __clang_major__ && 6 <= __clang_minor__
 // python/vcsn_cxx.cc: In constructor
 //   'minimize::minimize(const boost::python::detail::keywords<nkeywords>&,
 //                       const char*)':
@@ -21,11 +19,14 @@
 //   locally defined but not used [-Wunused-local-typedefs]
 //
 // BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(minimize, minimize, 0, 1);
-# pragma GCC diagnostic ignored "-Wunused-local-typedefs"
+#pragma clang diagnostic ignored "-Wunused-local-typedefs"
+
+#if defined __clang__
+# if 5 <= __clang_major__
+#  pragma clang diagnostic ignored "-Wzero-as-null-pointer-constant"
 # endif
 #elif defined __GNUC__
 # pragma GCC diagnostic ignored "-Wmissing-declarations"
-# pragma GCC diagnostic ignored "-Wunused-local-typedefs"
 # pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
 #endif
 
