@@ -1,4 +1,5 @@
 from difflib import unified_diff as diff
+import atexit
 import inspect
 import os
 import re
@@ -6,10 +7,20 @@ import sys
 
 import vcsn
 
-count = 0
+ntest = 0
 npass = 0
 nfail = 0
 
+def num_fail():
+    return nfail
+
+def num_pass():
+    return npass
+
+def num_test():
+    return ntest
+
+
 # For build-checks, use our abs_srcdir from tests/bin/vcsn. For
 # install checks, since the latter is not run (it runs
 # /usr/local/bin/vcsn), use Automake's srcdir.
@@ -20,15 +31,35 @@ srcdir = os.environ['abs_srcdir'] if 'abs_srcdir' in os.environ \
 medir = sys.argv[0].replace(".py", ".dir")
 
 
+def set_medir(dir):
+    global medir
+    medir = dir
+
+
+def mefile(fn, ext=None):
+    '''The path to the test's file `fn.ext`,
+    where `ext` is possibly empty.'''
+    return medir + '/' + fn + ('.' + ext if ext else '')
+
+def metext(fn, ext=None):
+    '''The content of the test's file `fn.ext`,
+    where `ext` is possibly empty.'''
+    return open(mefile(fn, ext)).read().strip()
+
+def meaut(fn, ext=None):
+    '''The automaton stored in the test's file `fn.ext`,
+    where `ext` is possibly empty.'''
+    return vcsn.automaton(filename=mefile(fn, ext))
+
+
 # http://stackoverflow.com/questions/377017/test-if-executable-exists-in-python
 def which(program):
     'Return the file name for program if it exists, None otherwise.'
-    import os
 
     def is_exe(fpath):
         return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
 
-    fpath, fname = os.path.split(program)
+    fpath, _ = os.path.split(program)
     if fpath:
         if is_exe(program):
             return program
@@ -75,7 +106,6 @@ def rst_diff(expected, effective):
 
 def load(fname):
     "Load the library automaton file `fname`."
-    import vcsn
     return vcsn.automaton(filename=vcsn.datadir + "/" + fname)
 
 
@@ -88,17 +118,23 @@ def here():
     return finfo.filename + ":" + str(finfo.lineno)
 
 
+def automaton(a):
+    if not isinstance(a, vcsn.automaton):
+        a = vcsn.automaton(a)
+    return a
+
+
 def FAIL(*msg, **kwargs):
-    global count, nfail
-    count += 1
+    global ntest, nfail # pylint: disable=global-statement
+    ntest += 1
     nfail += 1
     # Don't display multi-line failure messages, only the first line
     # will be reported anyway by the TAP driver.
     m = ' '.join(msg)
     if m.count("\n") == 0:
-        print('not ok ', count, m)
+        print('not ok', ntest, m)
     else:
-        print('not ok ', count)
+        print('not ok', ntest)
     loc = kwargs['loc'] if 'loc' in kwargs and kwargs['loc'] else here()
     print(loc + ": fail:", *msg)
     print()
@@ -106,10 +142,10 @@ def FAIL(*msg, **kwargs):
 
 
 def PASS(*msg, **kwargs):
-    global count, npass
-    count += 1
+    global ntest, npass # pylint: disable=global-statement
+    ntest += 1
     npass += 1
-    print('ok ', count, *msg)
+    print('ok ', ntest, *msg)
     loc = kwargs['loc'] if 'loc' in kwargs and kwargs['loc'] else here()
     print(loc + ": pass:", *msg)
     print()
@@ -126,12 +162,21 @@ def XFAIL(fun, exp=None):
     try:
         fun()
     except RuntimeError as e:
-        if exp is None or exp in str(e):
+        # Error messages about files that we read will have $srcdir as
+        # prefix.  Remove it.  E.g., from
+        #   while reading automaton: ../../tests/python/efsm.dir/bad_final_weight.efsm
+        # to
+        #   while reading automaton: efsm.dir/bad_final_weight.efsm
+        eff = str(e).replace(medir + '/', '')
+        if (exp is None
+            or isinstance(exp, re._pattern_type) and re.match(exp, eff)
+            or isinstance(exp, str) and exp in eff):
             PASS()
         else:
-            FAIL('does not include the expected error message')
+            FAIL('Unexpected error message')
             rst_file("Expected error", exp)
-            rst_file("Effective error", str(e))
+            rst_file("Effective error", eff)
+            rst_diff(exp, str(e))
     else:
         FAIL('did not raise an exception', str(fun))
 
@@ -145,15 +190,27 @@ def CHECK(effective, msg='', loc=None):
 
 
 def CHECK_EQ(expected, effective, loc=None):
-    "Check that `effective` is equal to `expected`."
+    'Check that `effective` is equal to `expected`.'
+    aut = isinstance(effective, vcsn.automaton)
     if isinstance(expected, str) and not isinstance(effective, str):
         effective = str(effective)
     if expected == effective:
         PASS(loc=loc)
     else:
+        msg = 'Unexpected result'
+        if aut:
+            expaut = automaton(expected)
+            effaut = automaton(effective)
+            if not effaut.is_accessible():
+                msg += (" (different, but cannot check whether isomorphic"
+                        ", as is not accessible)")
+            elif expaut.is_isomorphic(effaut):
+                msg += ' (different but isomorphic)'
+            else:
+                msg += ' (different and not even isomorphic)'
         exp = format(expected)
         eff = format(effective)
-        FAIL("Unexpected result", loc=loc)
+        FAIL(msg, loc=loc)
         rst_file("Expected output", exp)
         rst_file("Effective output", eff)
         rst_diff(exp, eff)
@@ -188,33 +245,50 @@ def normalize(a):
 
 def can_test_equivalence(a):
     ctx = a.context().format('sname')
-    expressionset_ws = re.match('.*expressionset<.*>', ctx) is None
-    return not(ctx.endswith('min')) and expressionset_ws
+    # Cannot check on Zmin and the like.
+    if ctx.endswith('min'):
+        return False
+    # Cannot check on expressionset as weightset.
+    if 'expressionset<' in ctx:
+        return False
+    # Cannot check equivalence on labelset.
+    if 'lat<' in ctx:
+        return False
+    return True
 
+def shortest(e, num=20):
+    if isinstance(e, vcsn.automaton):
+        e = e.proper()
+    return e.shortest(num)
 
 def CHECK_EQUIV(a1, a2):
-    '''Check that `a1` and `a2` are equivalent.  Also works for
-    expressions.'''
-    num = 10
+    '''Check that `a1` and `a2` are equivalent.  Works for
+    two automata, or two expressions.'''
+    # If we cannot check equivalence, check equality of the `num`
+    # shortest monomials.
+    num = 20
     # Cannot compute equivalence on Zmin, approximate with shortest.
     try:
         if can_test_equivalence(a1) and can_test_equivalence(a2):
             res = a1.is_equivalent(a2)
+            via = '(via is_equivalent)'
         else:
-            res = a1.proper().shortest(num) == a2.proper().shortest(num)
+            res = shortest(a1, num) == shortest(a2, num)
+            via = '(via shortests)'
     except RuntimeError as e:
         FAIL("cannot check equivalence: " + str(e))
         res = False
+        via = ''
 
     if res:
         PASS()
     else:
-        FAIL("not equivalent")
+        FAIL("not equivalent", via)
         rst_file("Left", format(a1))
         rst_file("Right", format(a2))
         try:
-            s1 = a1.proper().shortest(num).format('list')
-            s2 = a2.proper().shortest(num).format('list')
+            s1 = shortest(a1, num).format('list')
+            s2 = shortest(a2, num).format('list')
             rst_file("Left shortest", s1)
             rst_file("Right shortest", s2)
             rst_diff(s1, s2)
@@ -247,9 +321,12 @@ def CHECK_IS_EPS_ACYCLIC(a):
 
 def PLAN():
     "TAP requires that we announce the plan: the number of tests."
-    print('1..' + str(count))
+    print('1..' + str(ntest))
     print('PASS:', npass)
     print('FAIL:', nfail)
 
-import atexit
+def weightset_of(ctx):
+    s = str(ctx).lower().split('-> ')
+    return s[-1] if 0 < len(s) else None
+
 atexit.register(PLAN)
