@@ -9,21 +9,31 @@ from test import *
 ## -------------- ##
 
 # Check invalid input.
-def xfail(a):
-    XFAIL(lambda: vcsn.automaton(a))
+def xfail(a, *args):
+    XFAIL(lambda: vcsn.automaton(a), *args)
 
 # Syntax error: missing }.
 xfail(r'''digraph
 {
   vcsn_context = "lal_char(a), b"
-''')
+''', r'''4.1: syntax error, unexpected end
+
+^
+  while reading automaton''')
 
 # Syntax error: string not closed.
 xfail(r'''digraph
 {
   vcsn_context = "lal_char(a), b
 }
-''')
+''', r'''3.18-5.0: unexpected end of file in a string
+  vcsn_context = "lal_char(a), b
+                 ^^^^^^^^^^^^^^^
+3.18-5.0: invalid weightset name: "b\n}"
+  while reading context: "lal_char(a), b\n}\n"
+  vcsn_context = "lal_char(a), b
+                 ^^^^^^^^^^^^^^^
+  while reading automaton''')
 
 # Syntax error: attributes are assignments.
 xfail(r'''digraph
@@ -31,7 +41,10 @@ xfail(r'''digraph
   vcsn_context = "lal_char(a), b"
   a [attribute]
 }
-''')
+''', r'''4.15: syntax error, unexpected ], expecting =
+  a [attribute]
+              ^
+  while reading automaton''')
 
 # Syntax error: attributes are assignments.
 xfail(r'''digraph
@@ -39,7 +52,10 @@ xfail(r'''digraph
   vcsn_context = "lal_char(a), b"
   a [attribute =]
 }
-''')
+''', r'''4.17: syntax error, unexpected ], expecting identifier
+  a [attribute =]
+                ^
+  while reading automaton''')
 
 # Syntax error: comma used after empty attribute.
 xfail(r'''digraph
@@ -47,7 +63,10 @@ xfail(r'''digraph
   vcsn_context = "lal_char(a), b"
   a [,a=a]
 }
-''')
+''', r'''4.6: syntax error, unexpected ",", expecting ] or identifier
+  a [,a=a]
+     ^
+  while reading automaton''')
 
 # Syntax error: semicolon used after empty attribute
 xfail(r'''digraph
@@ -55,7 +74,10 @@ xfail(r'''digraph
   vcsn_context = "lal_char(a), b"
   a [;a=a]
 }
-''')
+''', r'''4.6: syntax error, unexpected ;, expecting ] or identifier
+  a [;a=a]
+     ^
+  while reading automaton''')
 
 # Syntax error: semicolon used after empty attribute
 xfail(r'''digraph
@@ -63,7 +85,10 @@ xfail(r'''digraph
   vcsn_context = "lal_char(a), b"
   a [a=a,;]
 }
-''')
+''', r'''4.10: syntax error, unexpected ;, expecting ] or identifier
+  a [a=a,;]
+         ^
+  while reading automaton''')
 
 # Invalid label: letter not in alphabet.
 xfail(r'''digraph
@@ -73,7 +98,11 @@ xfail(r'''digraph
   1 -> F1
   I0 -> 0
 }
-''')
+''', r'''4.10-20: invalid label: unexpected a
+  while reading: "a"
+  0 -> 1 [label = a]
+         ^^^^^^^^^^^
+  while reading automaton''')
 
 # Invalid label: \e is not valid in LAL.
 xfail(r'''digraph
@@ -83,7 +112,11 @@ xfail(r'''digraph
   1 -> F1
   I0 -> 0
 }
-''')
+''', r'''4.10-24: {a}: cannot represent \e
+  while reading: "\\e"
+  0 -> 1 [label = "\\e"]
+         ^^^^^^^^^^^^^^^
+  while reading automaton''')
 
 # Invalid label: aa is not valid in LAL.
 xfail(r'''digraph
@@ -93,7 +126,11 @@ xfail(r'''digraph
   1 -> F1
   I0 -> 0
 }
-''')
+''', r'''4.10-23: Poly[{a} -> B]: unexpected trailing characters: "a"
+  while reading: "aa"
+  0 -> 1 [label = "aa"]
+         ^^^^^^^^^^^^^^
+  while reading automaton''')
 
 # Invalid label: missing '>'.
 xfail(r'''digraph
@@ -103,7 +140,11 @@ xfail(r'''digraph
   1 -> F1
   I0 -> 0
 }
-''')
+''', r'''4.10-23: missing ">" after "<2"
+  while reading: "<2"
+  0 -> 1 [label = "<2"]
+         ^^^^^^^^^^^^^^
+  while reading automaton''')
 
 # No context defined (see the typo in vcsn_context).
 xfail(r'''digraph
@@ -113,7 +154,10 @@ xfail(r'''digraph
   1 -> F1
   I0 -> 0
 }
-''')
+''', r'''4.3: no vcsn_context defined
+  0 -> 1 [label = a]
+  ^
+  while reading automaton''')
 
 # Invalid context.
 xfail(r'''digraph
@@ -123,7 +167,25 @@ xfail(r'''digraph
   1 -> F1
   I0 -> 0
 }
-''')
+''', r'''3.18-26: invalid labelset name: "unknown"
+  while reading context: "unknown"
+  vcsn_context = "unknown"
+                 ^^^^^^^^^
+4.3: no vcsn_context defined
+  0 -> 1 [label = a]
+  ^
+  while reading automaton''')
+
+# Invalid context.
+xfail(r'''digraph
+{
+  vcsn_context = "lal, unknown"
+}
+''', r'''3.18-31: invalid weightset name: "unknown"
+  while reading context: "lal, unknown"
+  vcsn_context = "lal, unknown"
+                 ^^^^^^^^^^^^^^
+  while reading automaton''')
 
 # Invalid initial label.
 xfail(r'''digraph
@@ -133,7 +195,10 @@ xfail(r'''digraph
   1 -> F1
   I0 -> 0 [label = a]
 }
-''')
+''', r'''6.11-21: edit_automaton: invalid initial entry: a
+  I0 -> 0 [label = a]
+          ^^^^^^^^^^^
+  while reading automaton''')
 
 # Invalid final label.
 xfail(r'''digraph
@@ -143,7 +208,10 @@ xfail(r'''digraph
   1 -> F1 [label = a]
   I0 -> 0
 }
-''')
+''', r'''5.11-21: edit_automaton: invalid final entry: a
+  1 -> F1 [label = a]
+          ^^^^^^^^^^^
+  while reading automaton''')
 
 # \e makes no sense when not in lan.
 xfail(r'''digraph
@@ -151,7 +219,28 @@ xfail(r'''digraph
   vcsn_context = "lal_char(\\e), b"
   0 -> 1 [label = "\\e"]
 }
-''')
+''', r'''3.18-35: get_char: invalid escape: "\\e" in "\\e)>, b>"
+  while reading context: "context<letterset<char_letters(\\e)>, b>"
+  vcsn_context = "lal_char(\\e), b"
+                 ^^^^^^^^^^^^^^^^^^
+4.3: no vcsn_context defined
+  0 -> 1 [label = "\\e"]
+  ^
+  while reading automaton''')
+
+# There are spaces before 'digraph'.
+CHECK_EQ(vcsn.automaton(r'''digraph
+{
+  vcsn_context = "lal_char(abcd), b"
+  0 -> 0 [label="a, b, c, d"]
+}'''),
+vcsn.automaton('''
+\t  digraph
+\t  {
+\t    vcsn_context = "lal_char(abcd), b"
+\t    0 -> 0 [label="a, b, c, d"]
+\t  }
+'''))
 
 # An open context (letters are not specified).
 CHECK_EQ(vcsn.automaton(r'''digraph
@@ -214,14 +303,14 @@ vcsn.automaton(r'''digraph
 }'''))
 
 # A dot file which uses the HTML strings.  And a subgraph.
-CHECK_EQ(r'''context = "nullableset<letterset<char_letters(ab)>>, b"
+CHECK_EQ(r'''context = nullableset<letterset<char_letters(ab)>>, q
 $ -> 0
 $ -> 3
 0 -> 1 a, b
 1 -> $
-2 -> 1 a
+2 -> 1 <123>a
 3 -> 2 b''',
-         vcsn.automaton(filename=medir + '/html.gv').format('daut'))
+         meaut('html.gv').format('daut'))
 
 
 ## --------------- ##
@@ -410,7 +499,7 @@ CHECK_EQ('''digraph
   I0 -> 0
   0 -> F0
 }''',
-         vcsn.context('lal_char, b').expression('\z{c}').derived_term().dot())
+         vcsn.context('lal_char, b').expression(r'\z{c}').derived_term().dot())
 
 ## ------------- ##
 ## dot: simple.  ##
@@ -418,7 +507,7 @@ CHECK_EQ('''digraph
 
 ctx = vcsn.context('lal<string>, b')
 e = ctx.expression("'🍺':'🍾':'☕️':'🍷' & [^]*'🍺'[^]*'☕️'[^]* & ([^]*'🍷''🍾'[^]*){c}")
-CHECK_EQ(open(medir + '/drinks-simple.gv').read().strip(),
+CHECK_EQ(metext('drinks-simple.gv'),
          e.automaton().minimize().dot('simple'))
 
 
@@ -442,8 +531,8 @@ for fn in glob.glob(os.path.join(medir, '*.in.gv')):
 
 
 # Check state names in TikZ.
-a = vcsn.context('lal_char, b').expression('\e+a').derived_term()
-exp = open(os.path.join(medir, 'derived-term.tex')).read().strip()
+a = vcsn.context('lal_char, b').expression(r'\e+a').derived_term()
+exp = metext('derived-term.tex')
 CHECK_EQ(exp, a.format('tikz'))
 
 
@@ -456,10 +545,10 @@ ctx = vcsn.context('lal_char, q')
 a = ctx.expression('a*+(<2>a)*').derived_term(deterministic=True, lazy=True)
 a('aaa')
 
-exp = open(os.path.join(medir, 'lazy.out.gv')).read().strip()
+exp = metext('lazy.out.gv')
 CHECK_EQ(exp, a.format('dot'))
 
-exp = open(os.path.join(medir, 'lazy.tex')).read().strip()
+exp = metext('lazy.tex')
 CHECK_EQ(exp, a.format('tikz'))
 
 
@@ -468,17 +557,18 @@ CHECK_EQ(exp, a.format('tikz'))
 ## ----------- ##
 
 for fn in glob.glob(os.path.join(medir, '*.in.gv')):
+    print('Checking:', fn)
     a = vcsn.automaton(filename=fn)
 
-    # Check output.
+    # Check daut output.
     daut = a.format('daut')
-    exp = open(fn.replace('.in.gv', '.daut')).read().strip()
-    CHECK_EQ(exp, daut)
+    ref = open(fn.replace('.in.gv', '.daut')).read().strip()
+    CHECK_EQ(ref, daut)
 
-    # Check input: make sure we can read it.
-    CHECK_EQ(a, vcsn.automaton(exp, 'daut'))
-    CHECK_EQ(a, vcsn.automaton(exp, 'auto'))
-    CHECK_EQ(a, vcsn.automaton(exp))
+    # Check daut input: make sure we can read it.
+    CHECK_EQ(a, vcsn.automaton(ref, 'daut'))
+    CHECK_EQ(a, vcsn.automaton(ref, 'auto'))
+    CHECK_EQ(a, vcsn.automaton(ref))
 
 
 # A daut file whose names have quotes: beware of building "Ifoo" and
@@ -503,6 +593,63 @@ CHECK_EQ(r'''digraph
          vcsn.automaton('''$ -> "foo"
          "foo" -> $''', strip=False))
 
+# Check that "->" behaves as a keyword.
+CHECK_EQ(r'''digraph
+{
+  vcsn_context = "letterset<char_letters(a)>, b"
+  rankdir = LR
+  edge [arrowhead = vee, arrowsize = .6]
+  {
+    node [shape = point, width = 0]
+    I0
+    F1
+  }
+  {
+    node [shape = circle, style = rounded, width = 0.5]
+    0 [label = "0-0", shape = box]
+    1 [label = "1-1", shape = box]
+  }
+  I0 -> 0
+  0 -> 1 [label = "a"]
+  1 -> F1
+}''',
+         vcsn.automaton('''$-> 0-0
+0-0->1-1 a // Make sure this is not seen as a state "0-0->1-1".
+1-1 ->$''', strip=False))
+
+# Invalid transitions
+XFAIL(lambda: vcsn.automaton('''context = letterset<char_letters(abc)>, q
+$ -> 0 <a>
+0 -> $ <1/2>'''),
+      '''2.1-10: Q: invalid numerator: "a"
+  while reading: "a"
+  while reading: "<a>"
+$ -> 0 <a>
+^^^^^^^^^^
+  while reading automaton''')
+
+XFAIL(lambda: vcsn.automaton('''context = letterset<char_letters(abc)>, q
+$ -> 0 <1/2>
+0 -> $ <a>'''),
+      '''3.1-10: Q: invalid numerator: "a"
+  while reading: "a"
+  while reading: "<a>"
+0 -> $ <a>
+^^^^^^^^^^
+  while reading automaton''')
+
+XFAIL(lambda: vcsn.automaton('''context = letterset<char_letters(abc)>, q
+$ -> 0 <1/2>
+0 -> 1 <2/a>a
+1 -> $ <2>'''),
+      '''3.1-13: Q: invalid denominator: "a"
+  while reading: "2/a"
+  while reading: "<2/a>a"
+0 -> 1 <2/a>a
+^^^^^^^^^^^^^
+  while reading automaton''')
+
+
 ## ----------- ##
 ## I/O: FAdo.  ##
 ## ----------- ##
@@ -527,15 +674,46 @@ def check_fado(aut):
         SKIP("FAdo not installed")
 
 for fn in glob.glob(os.path.join(medir, '*.fado')):
-    exp = vcsn.automaton(filename=fn.replace('.fado', '.gv'))
+    ref = vcsn.automaton(filename=fn.replace('.fado', '.gv'))
     # Check that we can read FAdo.
-    CHECK_EQ(exp, vcsn.automaton(filename=fn, format='fado'))
-    CHECK_EQ(exp, vcsn.automaton(filename=fn, format='auto'))
+    CHECK_EQ(ref, vcsn.automaton(filename=fn, format='fado'))
+    CHECK_EQ(ref, vcsn.automaton(filename=fn, format='auto'))
 
     # Check that we can print FAdo.
     fado = open(fn).read().strip()
-    CHECK_EQ(fado, exp.format('fado'))
-    check_fado(a)
+    CHECK_EQ(fado, ref.format('fado'))
+    check_fado(ref)
+
+# Invalid kind
+XFAIL(lambda: vcsn.automaton('''@GFA 0 1 * 2 3
+  2 a 0
+  3 b 1''', 'fado'),
+      'fado: bad automaton kind in first line: @GFA')
+
+# Invalid initial states in DFA
+XFAIL(lambda: vcsn.automaton('''@DFA 0 1 * 2 3
+  2 a 0
+  3 b 1''', 'fado'),
+      'fado: invalid "*" for DFA in first line')
+
+# Multiple '*' in NFA
+XFAIL(lambda: vcsn.automaton('''@NFA 0 1 * 2 3 * 5 2
+  2 a 0
+  3 b 1''', 'fado'),
+      'fado: multiple "*" in first line')
+
+# Trailing characters in acceptor
+XFAIL(lambda: vcsn.automaton('''@NFA 0 1 * 2 3
+  2 a 0
+  3 b 1 c''', 'fado'),
+      'fado: unexpected trailing characters after: 3 b 1')
+
+# Epsilon in DFA
+XFAIL(lambda: vcsn.automaton('''@DFA 0 1
+  2 @epsilon 0
+  3 b 1''', 'fado'),
+      'fado: unexpected "@epsilon" in DFA, in: 2 @epsilon 0')
+
 
 ## --------------- ##
 ## Output: Grail.  ##
@@ -571,23 +749,23 @@ CHECK_EQ(vcsn.automaton('''context = "law_char, z"
   $ -> 0
   0 -> 1 a, b
   1 -> 1 c
-  1 -> $''', 'daut'),
+  1 -> $'''),
          vcsn.automaton('''context = "lal_char(abc), b"
   $ -> 0
   0 -> 1 a, b
   1 -> 1 c
-  1 -> $''', 'daut').automaton(vcsn.context("law_char(abc), z")))
+  1 -> $''').automaton(vcsn.context("law_char(abc), z")))
 
 # Convert an automaton to a smaller, valid, alphabet.
 CHECK_EQ(vcsn.automaton('''context = "law_char(abc), z"
-  0 -> 1 a, b''', 'daut'),
+  0 -> 1 a, b'''),
          vcsn.automaton('''context = "lal_char(a-z), b"
-  0 -> 1 a, b''', 'daut').automaton(vcsn.context("law_char(abc), z")))
+  0 -> 1 a, b''').automaton(vcsn.context("law_char(abc), z")))
 
 # Convert an automaton to a smaller, invalid, alphabet.
 XFAIL(lambda: vcsn.automaton('''context = "lal_char(abc), b"
-  0 -> 1 a, b''', 'daut').automaton(vcsn.context("law_char(xy), z")))
+  0 -> 1 a, b''').automaton(vcsn.context("law_char(xy), z")))
 
 # Convert to an invalid smaller weightset.
 XFAIL(lambda: vcsn.automaton('''context = "lal_char(abc), z"
-  0 -> 1 <3>a, b''', 'daut').automaton(vcsn.context("lal_char(xy), b")))
+  0 -> 1 <3>a, b''').automaton(vcsn.context("lal_char(xy), b")))
