@@ -3,8 +3,16 @@
 import vcsn
 from test import *
 
+def aut(ctx, exp, algo):
+    if isinstance(ctx, str):
+        ctx = vcsn.context(ctx)
+    return ctx.expression(exp).automaton(algo)
+
+def std(ctx, exp, algo='standard'):
+    return aut(ctx, exp, algo)
+
 def dt(ctx, exp):
-    return vcsn.context(ctx).expression(exp).derived_term()
+    return std(ctx, exp, 'expansion')
 
 ## ---------------------- ##
 ## Existing transitions.  ##
@@ -20,8 +28,8 @@ CHECK_EQ('a*a', a2.expression())
 ## (a+b)* & (b+c)* = b*.  ##
 ## ---------------------- ##
 
-lhs = vcsn.context('lal_char(ab), b').expression('(a+b)*').standard()
-rhs = vcsn.context('lal_char(bc), b').expression('(b+c)*').standard()
+lhs = std('lal_char(ab), b', '(a+b)*')
+rhs = std('lal_char(bc), b', '(b+c)*')
 CHECK_EQ('''digraph
 {
   vcsn_context = "letterset<char_letters(b)>, b"
@@ -50,8 +58,8 @@ CHECK_EQ('''digraph
 ## ab & cd = 0.  ##
 ## ------------- ##
 
-lhs = vcsn.context('lal_char(ab), b').expression('ab').standard()
-rhs = vcsn.context('lal_char(cd), b').expression('cd').standard()
+lhs = std('lal_char(ab), b', 'ab')
+rhs = std('lal_char(cd), b', 'cd')
 CHECK_EQ('''digraph
 {
   vcsn_context = "letterset<char_letters()>, b"
@@ -74,8 +82,8 @@ CHECK_EQ('''digraph
 ## (a+b)* & (c+d)* = \e.  ##
 ## ---------------------- ##
 
-lhs = vcsn.context('lal_char(ab), b').expression('(a+b)*').standard()
-rhs = vcsn.context('lal_char(cd), b').expression('(c+d)*').standard()
+lhs = std('lal_char(ab), b', '(a+b)*')
+rhs = std('lal_char(cd), b', '(c+d)*')
 CHECK_EQ('''digraph
 {
   vcsn_context = "letterset<char_letters()>, b"
@@ -195,11 +203,10 @@ def check(exp, aut):
     CHECK_EQ(exp, aut.expression())
 
 # RatE and B, in both directions.
-a1 = vcsn.context('lal_char(ab), seriesset<lal_char(uv), q>') \
-         .expression('(<u>a+<v>b)*').standard()
-a2 = vcsn.context('lal_char(ab), b').expression('a{+}').standard()
-check('<u>a(\e+<u>a(<u>a)*)', a1 & a2)
-check('<u>a(\e+<u>a(<u>a)*)', a2 & a1)
+a1 = std('lal_char(ab), seriesset<lal_char(uv), q>', '(<u>a+<v>b)*')
+a2 = std('lal_char(ab), b', 'a{+}')
+check(r'<u>a(\e+<u>a(<u>a)*)', a1 & a2)
+check(r'<u>a(\e+<u>a(<u>a)*)', a2 & a1)
 
 # Z, Q, R.
 z = dt('lal_char(ab), z', '(<2>a+<3>b)*')
@@ -222,17 +229,17 @@ check('(<0.1>a+<0.1>b)*', r & q)
 ## Non-commutative.  ##
 ## ----------------- ##
 
-a1 = vcsn.context('lal_char(ab), seriesset<lal_char(uv), q>') \
-         .expression('<u>a<v>b').standard()
-a2 = vcsn.context('lal_char(ab), seriesset<lal_char(xy), q>') \
-         .expression('<x>a<y>b').standard()
+a1 = std('lal_char(ab), seriesset<lal_char(uv), q>',
+         '<u>a<v>b')
+a2 = std('lal_char(ab), seriesset<lal_char(xy), q>',
+         '<x>a<y>b')
 
 def check_enumerate(exp, aut):
     CHECK_EQ(exp, aut.strip().shortest(len=4))
 
 check_enumerate('<uxvy>ab', a1 & a2)
-check_enumerate('\z', a1.transpose() & a2)
-check_enumerate('\z', a1 & a2.transpose())
+check_enumerate(r'\z', a1.transpose() & a2)
+check_enumerate(r'\z', a1 & a2.transpose())
 check_enumerate('<vyux>ba', a1.transpose() & a2.transpose())
 
 
@@ -277,7 +284,7 @@ digraph
 ctx = vcsn.context('lal_char(x), seriesset<lal_char(abcd), q>')
 a = dict()
 for l in ['a', 'b', 'c', 'd']:
-    a[l] = ctx.expression("<{}>x".format(l)).standard()
+    a[l] = std(ctx, '<{}>x'.format(l))
 check_enumerate('<abcd>x', a['a'] & a['b'] & a['c'] & a['d'])
 
 
@@ -292,7 +299,7 @@ qr = vcsn.context('lal_char(a), seriesset<lal_char(uv), q>') \
 z = vcsn.context('lal_char(b), z').expression('<2>b*')
 q = vcsn.context('lal_char(c), q').expression('<1/3>c*')
 r = vcsn.context('lal_char(d), r').expression('<.4>d*')
-CHECK_EQ('<u>a*&<<2>\e>b*&<<0.333333>\e>c*&<<0.4>\e>d*', qr & z & q & r)
+CHECK_EQ(r'<u>a*&<<2>\e>b*&<<0.333333>\e>c*&<<0.4>\e>d*', qr & z & q & r)
 
 ## ------------------------- ##
 ## polynomial & polynomial.  ##
@@ -305,30 +312,30 @@ c = poly('<3>c')
 CHECK_EQ('<4>a + <9>b', a & a)
 CHECK_EQ('<6>b', a & b)
 CHECK_EQ('<6>b', b & a)
-CHECK_EQ('\z', a & c)
+CHECK_EQ(r'\z', a & c)
 CHECK_EQ('<9>c', b & c)
 
 poly = vcsn.context('lan_char, q').polynomial
-a = poly('<2>\e')
-b = poly('<2>a+<3>\e')
+a = poly(r'<2>\e')
+b = poly(r'<2>a+<3>\e')
 c = poly('<2>a+<3>c')
-CHECK_EQ('<4>\e', a & a)
-CHECK_EQ('\z', a & c)
-CHECK_EQ('<6>\e', a & b)
+CHECK_EQ(r'<4>\e', a & a)
+CHECK_EQ(r'\z', a & c)
+CHECK_EQ(r'<6>\e', a & b)
 CHECK_EQ('<4>a', b & c)
 
 poly = vcsn.context('law_char, q').polynomial
 a = poly('<1>aa')
 b = poly('<2>ab')
-CHECK_EQ('\z', a & b)
+CHECK_EQ(r'\z', a & b)
 CHECK_EQ('aa', a & a)
 
 ## ----------------- ##
 ## nullable labels.  ##
 ## ----------------- ##
 
-lhs = vcsn.context('lan_char(ab), b').expression('(a+b)*').thompson()
-rhs = vcsn.context('lan_char(bc), b').expression('(b+c)*').thompson()
+lhs = aut('lan_char(ab), b', '(a+b)*', 'thompson')
+rhs = aut('lan_char(bc), b', '(b+c)*', 'thompson')
 res = r'''digraph
 {
   vcsn_context = "nullableset<letterset<char_letters(b)>>, b"
@@ -404,9 +411,9 @@ res = r'''digraph
 }'''
 CHECK_EQ(res, lhs & rhs)
 CHECK_EQUIV(vcsn.automaton(res),
-            vcsn.context("lal_char(b), b").expression("b*").standard())
+            std('lal_char(b), b', 'b*'))
 
-third = vcsn.context('lan_char(bcd), b').expression('(b+c+d)*').thompson()
+third = aut('lan_char(bcd), b', '(b+c+d)*', 'thompson')
 res = r'''digraph
 {
   vcsn_context = "nullableset<letterset<char_letters(b)>>, b"
@@ -626,57 +633,125 @@ res = r'''digraph
 }'''
 CHECK_EQ(res, lhs & rhs & third)
 CHECK_EQUIV(vcsn.automaton(res),
-            vcsn.context("lal_char(b), b").expression("b*").standard())
+            std('lal_char(b), b', 'b*'))
+
+
+# Simple micro-optimization: don't build states which have no possible
+# outgoing transitions.
+a1 = vcsn.automaton(r'''
+context = "lan, b"
+$ -> 0
+''')
+
+a2 = vcsn.automaton(r'''
+context = "lan, b"
+$ -> 0
+0 -> 1 \e
+1 -> $
+''')
+
+res = r'''digraph
+{
+  vcsn_context = "nullableset<letterset<char_letters()>>, b"
+  rankdir = LR
+  edge [arrowhead = vee, arrowsize = .6]
+  {
+    node [shape = point, width = 0]
+    I0
+  }
+  {
+    node [shape = circle, style = rounded, width = 0.5]
+    0 [label = "0, (0, !\\e)", shape = box, color = DimGray]
+  }
+  I0 -> 0 [color = DimGray]
+}'''
+CHECK_EQ(res, a1 & a2)
+
+
+
+# Because of stupid implementation details, we used to _not_ insplit
+# the automaton in repeated conjunction, yielding an incorrect result.
+a = vcsn.automaton(r'''
+context = "lan, q"
+$ -> 0
+0 -> 0 <1/2>\e, <2>a
+0 -> $
+''')
+CHECK_EQ((a & a).strip(), a & 2)
+CHECK_EQ((a & a & a).strip(), a & 3)
+
+
+# An automaton such that the insplit-based and the rank-based
+# algorithm give different results.
+a = vcsn.automaton(r'''
+context = "lan, q"
+$ -> 0
+0 -> 0 <1/2>\e, a
+0 -> $
+''')
+
+insplit_res = r'''digraph
+{
+  vcsn_context = "nullableset<letterset<char_letters(a)>>, q"
+  rankdir = LR
+  edge [arrowhead = vee, arrowsize = .6]
+  {
+    node [shape = point, width = 0]
+    I0
+    F0
+    F1
+    F2
+    F3
+  }
+  {
+    node [shape = circle, style = rounded, width = 0.5]
+    0 [label = "0, (0, !\\e), (0, !\\e)", shape = box]
+    1 [label = "0, (0, \\e), (0, !\\e)", shape = box]
+    2 [label = "0, (0, !\\e), (0, \\e)", shape = box]
+    3 [label = "0, (0, \\e), (0, \\e)", shape = box]
+  }
+  I0 -> 0
+  0 -> F0
+  0 -> 0 [label = "<1/2>\\e, a"]
+  0 -> 1 [label = "<1/2>\\e"]
+  0 -> 2 [label = "<1/2>\\e"]
+  1 -> F1
+  1 -> 0 [label = "a"]
+  1 -> 1 [label = "<1/2>\\e"]
+  1 -> 3 [label = "<1/2>\\e"]
+  2 -> F2
+  2 -> 0 [label = "a"]
+  2 -> 2 [label = "<1/2>\\e"]
+  3 -> F3
+  3 -> 0 [label = "a"]
+  3 -> 3 [label = "<1/2>\\e"]
+}'''
+
+CHECK_EQ(insplit_res, a & a & a)
+
+
+
 
 ###############################################
 ## Check mixed epsilon and letters going out ##
 ###############################################
 
+a1 = vcsn.automaton(r'''
+context = "lan_char(abc), b"
+  $ -> 0
+  0 -> 1 a
+  1 -> $
+  0 -> 2 \e
+  2 -> $
+''')
 
-a1 = vcsn.automaton(r'''digraph
-{
-  vcsn_context = "lan_char(abc), b"
-  rankdir = LR
-  {
-    node [shape = point, width = 0]
-    I0
-    F1
-    F5
-  }
-  {
-    node [shape = circle]
-    0
-    1
-    5
-  }
-  I0 -> 0
-  0 -> 1 [label = "a"]
-  1 -> F1
-  0 -> 5 [label = "\\e"]
-  5 -> F5
-}''')
-
-
-a2 = vcsn.automaton(r'''digraph
-{
-  vcsn_context = "lan_char(abc), b"
-  rankdir = LR
-  {
-    node [shape = point, width = 0]
-    I0
-    F1
-    F5
-  }
-  {
-    node [shape = circle]
-    0
-    5
-  }
-  I0 -> 0
-  0 -> 5 [label = "\\e"]
-  5 -> 6 [label = "a"]
-  6 -> F6
-}''')
+a2 = vcsn.automaton(r'''
+context = "lan_char(abc), b"
+  $ -> 0
+  0 -> 1 \e
+  1 -> 2 a
+  2 -> $
+''')
 
 res = r'''digraph
 {
@@ -703,7 +778,6 @@ res = r'''digraph
   2 -> 4 [label = "a"]
   4 -> F4
 }'''
-
 CHECK_EQ(res, a1 & a2)
 
 ## ------------- ##
@@ -711,6 +785,6 @@ CHECK_EQ(res, a1 & a2)
 ## ------------- ##
 
 # Check that the conjunction structure is transparent: invoke it.
-a = vcsn.context('lal_char(a), b').expression('a').standard()
+a = std('lal_char(a), b', 'a')
 a = a & a & a
 CHECK_EQ('1', a('a'))
