@@ -4,10 +4,10 @@ import vcsn
 from test import *
 
 # Eliminate state 0, or 1, or 2.
-a = vcsn.automaton(filename = medir + '/lao.gv')
+a = meaut('lao.gv')
 for i in [0, 1, 2]:
     print("Eliminating state", i)
-    exp = open(medir + '/lao-elim-{}.gv'.format(i)).read().strip()
+    exp = metext('lao-elim-{}.gv'.format(i))
     CHECK_EQ(exp, a.eliminate_state(i))
 
 XFAIL(lambda: a.eliminate_state(3))
@@ -15,7 +15,7 @@ XFAIL(lambda: a.eliminate_state(3))
 a = vcsn.B.expression('ab*c').standard().lift()
 for i in range(4):
     print("elimination:", i)
-    exp = open(medir + '/abc-elim-{}.gv'.format(i)).read().strip()
+    exp = metext('abc-elim-{}.gv'.format(i))
     a = a.eliminate_state(-1)
     CHECK_EQ(exp, a)
 
@@ -30,7 +30,7 @@ XFAIL(lambda: a.eliminate_state())
 # transitions (<\e>\e, and <aa*>$) instead of a unique one
 # (<\e+aa*>$), which is visible in the dot output.
 a = vcsn.context('lal_char, b').expression('aa*').standard().lift()
-CHECK_EQ(r'''context = "lao, expressionset<letterset<char_letters(a)>, b>"
+CHECK_EQ(r'''context = lao, expressionset<letterset<char_letters(a)>, b>
 $ -> 0
 0 -> 1 <a>
 1 -> $ <\e+aa*>''',
