@@ -4,6 +4,10 @@ import vcsn
 from test import *
 
 def check_mult(lhs, rhs):
+    '''Check l * r for l in lhs, r in rhs.
+    Check the different flavors: general, deterministic,
+    and standard.'''
+
     if isinstance(lhs, list):
         for aut in lhs:
             check_mult(aut, rhs)
@@ -147,6 +151,8 @@ digraph
 }
 '''), a.multiply(a))
 
+
+
 ## --------------------- ##
 ## Heterogeneous input.  ##
 ## --------------------- ##
@@ -160,7 +166,7 @@ def check(exp, eff):
 a1 = vcsn.context('lal_char(a), expressionset<lal_char(uv), b>') \
          .expression('<u>a').derived_term()
 a2 = vcsn.context('lal_char(b), b').expression('b*').standard()
-check('<u>a(\e+bb*)', a1*a2)
+check(r'<u>a(\e+bb*)', a1*a2)
 # FIXME: Why don't we get (\e+bb*)<u>a?
 check('<u>a+bb*<u>a', a2*a1)
 
@@ -189,30 +195,48 @@ br = vcsn.context('lal_char(a), expressionset<lal_char(uv), b>') \
 z = vcsn.context('lal_char(b), z').expression('<2>b')
 q = vcsn.context('lal_char(c), q').expression('<1/3>c')
 r = vcsn.context('lal_char(d), r').expression('<.4>d')
-CHECK_EQ('<u>a<<2>\e>b<<0.333333>\e>c<<0.4>\e>d', str(br * z * q * r))
+CHECK_EQ(r'<u>a<<2>\e>b<<0.333333>\e>c<<0.4>\e>d', str(br * z * q * r))
+
+# This is a real regression (#68): we used to interpret `<3><3>a` as
+# `<27>a` because `3 * 3` in Z was intrepreted as `3 ** 3`, because `w
+# * n` is interpreted as repeated multiplication, `w ** n` when n is
+# an int.  Which collides with binary multiplication in Z.
+zexp = vcsn.context('lal, z').expression
+CHECK_EQ(zexp('<9>a'), zexp('<3><3>a'))
 
 ## --------------- ##
 ## label * label.  ##
 ## --------------- ##
 
-c = vcsn.context('law_char, b')
-CHECK_EQ(c.label('abc'),
-         c.label('ab') * c.label('\e') * c.label('c'))
-
+l = vcsn.context('law_char, b').label
+CHECK_EQ(l('abc'),
+         l('ab') * l(r'\e') * l('c'))
+CHECK_EQ(l('\e'),
+         l('ab') ** 0)
+CHECK_EQ(l('ababab'),
+         l('ab') ** 3)
+c = vcsn.context('lan, q')
+c2 = c|c
+CHECK_EQ(c2.word('abc|aBc'),
+         c.word('a') * c2.word('b|B') * c.word('c'))
 
 ## ------------------------- ##
 ## polynomial * polynomial.  ##
 ## ------------------------- ##
 
-c = vcsn.context('law_char, z')
-CHECK_EQ(c.polynomial('c + <5>d + <2>ac + <10>ad + <3>bc + <15>bd'),
-         c.polynomial('\e + <2>a + <3>b') * c.polynomial('\e') * c.polynomial('c + <5>d'))
+pol = vcsn.context('law_char, z').polynomial
+CHECK_EQ(pol('c + <5>d + <2>ac + <10>ad + <3>bc + <15>bd'),
+         pol(r'\e + <2>a + <3>b') * pol(r'\e') * pol('c + <5>d'))
 
 
 ## ----------------- ##
 ## weight * weight.  ##
 ## ----------------- ##
 
-c = vcsn.context('lal_char, seriesset<lal_char, q>')
-CHECK_EQ(c.weight('<4>aa+<6>ab+<6>ba+<9>bb'),
-         c.weight('<2>a+<3>b') * c.weight('<2>a+<3>b'))
+w = vcsn.context('lal_char, seriesset<lal_char, q>').weight
+CHECK_EQ(w('<4>aa+<6>ab+<6>ba+<9>bb'),
+         w('<2>a+<3>b') * w('<2>a+<3>b'))
+# See the comment above about #68.
+w = vcsn.context('lal, z').weight
+CHECK_EQ(w('9'), w('3') * w('3'))
+# More * and ** tests in tests/python/weight.py.
