@@ -13,24 +13,24 @@ std::ostream& operator<<(std::ostream& o, std::tuple<Args...>& args)
 
 int main()
 {
-  std::map<char, int> ints
-    = { {'1', 1},
-        {'2', 20},
-        {'3', 30},
-        {'4', 40} };
-  std::map<char, double> floats
-    = { // {'1', 1.1},
-        {'2', 2.22},
-        {'3', 3.3}
-        //, {4, 4.4}
-  };
-  std::map<char, std::string> strings
-    = { {'1', "one"},
-        {'2', "deux"},
-        {'3', "three"},
-        {'4', "four"} };
+  auto ints = std::map<char, int>
+    { {'1', 1},
+      {'2', 20},
+      {'3', 30},
+      {'4', 40} };
+  auto floats = std::map<char, double>
+    { // {'1', 1.1},
+      {'2', 2.22},
+      {'3', 3.3}
+      //, {4, 4.4}
+    };
+  auto strings = std::map<char, std::string>
+    { {'1', "one"},
+      {'2', "deux"},
+      {'3', "three"},
+      {'4', "four"} };
 
-  for (auto i: vcsn::zip_maps<vcsn::as_tuple>(ints, floats, strings))
+  for (const auto& i: vcsn::zip_maps<vcsn::as_tuple>(ints, floats, strings))
     std::cout << std::get<0>(i).first << ':'
               << '('
               << std::get<0>(i).second
@@ -38,6 +38,5 @@ int main()
               << std::get<1>(i).second
               << ", "
               << std::get<2>(i).second
-              << ')'
-              << std::endl;
+              << ")\n";
 }
